"use strict";

var JPromise = Promise || require("bluebird");

/**
 * A Jefferson proxy that resolves promise-based middleware functions.
 * If a middleware function accepts 2 arguments, we will wrap it in a promise chain.
 * If it throws or resolves to an error, next() is invoked with the error.
 * @type {{name: string, init: Function}}
 */
module.exports = {
    name: "Promise Handler",
    init: function (delegate, conf) {
        return function (req, res, next) {
            var nextTriggered = false;
            var invokeNext = function (arg) {
                if (!nextTriggered) {
                    nextTriggered = true;
                    next(arg);
                }
            };
            var isFlowHalted = function () {
                return conf && conf.haltCondition && conf.haltCondition(req, res);
            };
            var possiblePromise = delegate(req, res, invokeNext);
            return JPromise.resolve(possiblePromise).then(function () {
                if (!isFlowHalted()) {
                    invokeNext();
                }
            })["catch"](next);
        };
    }
};